package acube.framework.captcha;

import java.util.Locale;

import org.jdom.Element;
import acube.framework.captcha.exception.CaptchaAcubeException;


/**
 * 
 * This is a model of a captcha service. All captcha services use into
 * the Acube Framework have to extends this class. In order to use
 * captcha services of JCaptcha projet, you have to create a
 * CaptchaService in Acube Framework. This Acube Captcha service will
 * contain the id [{@link CaptchaAbstractService#octoCaptchaServiceId}]
 * and the instance [{@link CaptchaAbstractService#octoCaptchaServiceInstance}]
 * corresponding to the service of JCaptcha projet. <br/><br/> The
 * Acube Framework is able to support several captcha service by a
 * simple configuration: <br/> <code>
 &lt;root id="captcha-config"&gt;<br/>
 &nbsp;&nbsp;&lt;captcha name="captcha1" id="1" class="acube.framework.captcha.
 captchaServices.Captcha1Service"&gt;<br/>
 &nbsp;&nbsp;&nbsp;&nbsp;&lt;bean id="captcha1" class="com.octo.captcha.service
 ...."/&gt;<br/>
 &nbsp;&nbsp;&lt;/captcha&gt;<br/>
 &nbsp;&nbsp;&lt;captcha name="captcha2" id="2" class="acube.framework.captcha.
 captchaServices.Captcha2Service"&gt;<br/>
 &nbsp;&nbsp;&nbsp;&nbsp;&lt;bean id="imageCaptchaService2" class="com.octo.
 captcha.service..."/&gt;<br/>
 &nbsp;&nbsp;&lt;/captcha&gt;<br/>
 &lt;/root&gt;<br/><br/>
 </code>
 * explanations :
 * <ul>
 * <li>2 captcha tags:</li>
 * <ul>
 * By example.
 * </ul>
 * <li>Each id'value are differents; Each name's value are
 * differents:</li>
 * <ul>
 * It necessary that all "id" or "name" attribute are unique.<br/>
 * <u>id :</u> &lt;captcha&gt; tag's "id" attribute refers switch
 * number in
 * {@link CaptchaManager#CaptchaSelection(String) CaptchaSelection method}.
 * Important : When you add a new captcha in file conf, you have to
 * update the CaptchaSelection(String) with the id number.<br/>
 * <u>name :</u> It 's the name of your captcha. It is used par the
 * service to generate the image or compute the reponse of user.
 * </ul>
 * <li>&lt;captcha&gt; 's "class" attribute refers an Acube class:</li>
 * <ul>
 * This class refers the Acube 's captcha service class. This class
 * must extends {@link CaptchaAbstractService}
 * </ul>
 * <li>&lt;bean&gt; 's "class" attribute refers a JCaptcha class:</li>
 * <ul>
 * It corresponds to the service of JCaptcha you want to use.
 * </ul>
 * </ul>
 * @date 20 dc. 07
 * @author EDS - bzgn4l
 */
public abstract class CaptchaAbstractService {

    /**
     * <code>id</code> the id refers Id attribute in
     * <code>&lt;captcha&gt;</code> tag.
     */
    private Integer id;

    /**
     * <code>ContentType</code> the ContentType refers http servlet
     * response 's content type.
     */
    private String contentType;

    /**
     * <code>octoCaptchaServiceInstance</code> the
     * octoCaptchaServiceInstance refers JCaptcha Service instance.
     */
    private Object octoCaptchaServiceInstance;

    /**
     * <code>octoCaptchaServiceId</code> the octoCaptchaServiceId
     * refers id of JCaptcha Service instance defined in
     * <code>&lt;captcha&gt;</code> tag.
     */
    private String octoCaptchaServiceId;

    /**
     * CaptchaAbstractService's Constructor.
     */
    public CaptchaAbstractService() {

    };

    /**
     * This method create a captcha 's new file output stream.
     * @param idCaptcha
     *            id of captcha 's new file output stream
     * @param lLocale
     *            refers Locales of http Servlet Response.
     * @return the file in array of bytes.
     * @throws CaptchaAcubeException
     *             CaptchaAcubeException
     */
    public abstract byte[] getOutStream(
            String idCaptcha, Locale lLocale)
    throws CaptchaAcubeException;

    /**
     * The {@link CaptchaManagerLoader CaptchaManagerLoader} parse the
     * root config.<br/> For each reading of &lt;/captcha&gt; tags,
     * the CaptchaManagerLoader can select your AcubeCaptchaService
     * class for your specific jcaptcha service.<br/> So, The
     * {@link CaptchaManagerLoader CaptchaManagerLoader} can call this
     * method (on starting) in order to parse every &lt;/bean&gt; you
     * have defined in the captcha configuration file.<br/> This
     * method that you have to implement must finish calls of
     * {@link CaptchaAbstractService#setOctoCaptchaServiceInstance(String)}
     * and
     * {@link CaptchaAbstractService#setOctoCaptchaServiceId(String)}
     * @param eCaptcha
     *            refers captcha Element (jdom) of &lt;/captcha&gt;
     * @throws CaptchaAcubeException
     *             CaptchaAcubeException
     */
    public abstract void parseElement(
            Element eCaptcha) throws CaptchaAcubeException;

    /**
     * It the resolve method of your captcha.
     * @param idCaptcha
     *            id of captcha 's new file output stream
     * @param sResponse
     *            user response
     * @return boolean true where it solved else false
     * @throws CaptchaAcubeException
     *             CaptchaAcubeException
     */
    public abstract boolean computeValidation(
            String idCaptcha, String sResponse) throws CaptchaAcubeException;

    /**
     * @return the compatible content type for your catpcha type of
     *         file.
     */
    public final String getContentType() {

        return this.contentType;
    }

    /**
     * Set a compatible http servlet response content type for your
     * catpcha type of file.
     * @param contentType  : the http servlet response content type.
     */
    public void setContentType(
            String contentType) {

        this.contentType = contentType;
    }

    /**
     * @return the instance of JCaptcha Service (the class you have
     *         defined in conf file for &lt;/bean&gt; tag).
     */
    public final Object getOctoCaptchaServiceInstance() {

        return this.octoCaptchaServiceInstance;
    }

    /**
     * Must be call by your specific 
     * {@link CaptchaAbstractService#parseElement(Element)}.
     * @param octoCaptchaServiceInstance :
     *            the instance of the class you have defined in conf
     *            file for &lt;/bean&gt; tag.
     */
    public final void setOctoCaptchaServiceInstance(
            Object octoCaptchaServiceInstance) {

        this.octoCaptchaServiceInstance = octoCaptchaServiceInstance;
    }

    /**
     * @return the "id" of JCaptcha Service that you have defined in
     *         conf file for &lt;/bean&gt; tag.
     */
    public final String getOctoCaptchaServiceId() {

        return this.octoCaptchaServiceId;
    }

    /**
     * Must be call by your specific
     * {@link CaptchaAbstractService#parseElement(Element)}.
     * @param octoCaptchaServiceId :
     *            the "id" of JCaptcha Service that you have defined
     *            in conf file for &lt;/bean&gt; tag.
     */
    public final void setOctoCaptchaServiceId(
            String octoCaptchaServiceId) {

        this.octoCaptchaServiceId = octoCaptchaServiceId;
    }

    /**
     * @return the "id" of Acube Captcha Service that you have defined
     *         in conf file for &lt;/captcha&gt; tag.
     */
    public final Integer getId() {

        return this.id;
    }

    /**
     * Setter of CaptchaServiceID.
     * @param id :
     *            the "id" of Acube Captcha Service that you have
     *            defined in conf file &lt;/captcha&gt; tag.
     */
    public final void setId(
            Integer id) {

        this.id = id;
    }
}
