/*
 * Copyright 1999,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package acube.framework.clavierVirtuel.realm;

import java.io.File;
import java.security.Principal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.apache.catalina.LifecycleException;
import org.apache.catalina.realm.Constants;
import org.apache.catalina.realm.GenericPrincipal;
import org.apache.catalina.util.StringManager;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tomcat.util.digester.Digester;

/**
 * Simple implementation of <b>Realm</b> that reads an XML file to
 * configure the valid users, passwords, and roles. The file format
 * (and default file location) are identical to those currently
 * supported by Tomcat 3.X.
 * <p>
 * <strong>IMPLEMENTATION NOTE</strong>: It is assumed that the
 * in-memory collection representing our defined users (and their
 * roles) is initialized at application startup and never modified
 * again. Therefore, no thread synchronization is performed around
 * accesses to the principals collection.
 * 
 * @author Craig R. McClanahan
 * @version $Revision: 373023 $ $Date: 2006-01-27 17:17:43 -0600 (Fri,
 *          27 Jan 2006) $
 */

public class MemoryRealm extends RealmBaseAcube {

    /**
     * Descriptive information about this Realm implementation.
     */

    protected static final String NAME = "MemoryRealm";

    /**
     * <code>log</code> the log for this realm
     */
    private static Log log = LogFactory.getLog(MemoryRealm.class);

    // ----------------------------------------------------- Instance
    // Variables

    /**
     * The Digester we will use to process in-memory database files.
     */
    private static Digester digester = null;

    /**
     * The string manager for this package.
     */
    private static StringManager sm =
            StringManager.getManager(Constants.Package);

    /**
     * Descriptive information about this Realm implementation.
     */
    private final String info = "org.apache.catalina.realm.MemoryRealm/1.0";

    /**
     * The pathname (absolute or relative to Catalina's current
     * working directory) of the XML file containing our database
     * information.
     */
    private String pathname = "conf/tomcat-users.xml";

    /**
     * The set of valid Principals for this Realm, keyed by user NAME.
     */
    private Map<String, GenericPrincipal> principals =
            new HashMap<String, GenericPrincipal>();

    // -------------------------------------------------------------
    // Properties

    /**
     * Return descriptive information about this Realm implementation
     * and the corresponding version number, in the format
     * <code>&lt;description&gt;/&lt;version&gt;</code>.
     * 
     * @return descriptive information about this Realm
     */
    public String getInfo() {

        return this.info;

    }

    /**
     * Return the pathname of our XML file containing user
     * definitions.
     * 
     * @return the pathname of our XML file containing user
     *         definitions
     */
    public String getPathname() {

        return this.pathname;

    }

    /**
     * Set the pathname of our XML file containing user definitions.
     * If a relative pathname is specified, it is resolved against
     * "catalina.base".
     * 
     * @param pathname
     *            The new pathname
     */
    public void setPathname(
            String pathname) {

        this.pathname = pathname;

    }

    // ---------------------------------------------------------
    // Public Methods

    /**
     * Return the Principal associated with the specified username and
     * credentials, if there is one; otherwise return
     * <code>null</code>.
     * 
     * @param username
     *            Username of the Principal to look up
     * @param credentials
     *            Password or other credentials to use in
     *            authenticating this username
     * @return the Principal associated with the specified username
     *         and credentials
     */
    public Principal authenticate(
            String username, String credentials) {

        GenericPrincipal principal =
                (GenericPrincipal) this.principals.get(username);

        boolean validated = false;
        if (principal != null) {
            if (this.hasMessageDigest()) {
                // Hex hashes should be compared case-insensitive
                validated =
                        (this.digest(credentials).equalsIgnoreCase(principal
                                .getPassword()));
            } else {
                validated =
                        (this.digest(credentials).equals(principal
                                .getPassword()));
            }
        }

        if (validated) {
            if (MemoryRealm.log.isDebugEnabled()) {
                MemoryRealm.log.debug(MemoryRealm.sm.getString(
                    "memoryRealm.authenticateSuccess", username));
            }

            return (principal);
        } else {
            if (MemoryRealm.log.isDebugEnabled()) {
                MemoryRealm.log.debug(MemoryRealm.sm.getString(
                    "memoryRealm.authenticateFailure", username));
            }
            return (null);
        }

    }

    // --------------------------------------------------------
    // Package Methods

    /**
     * Add a new user to the in-memory database.
     * 
     * @param username
     *            User's username
     * @param password
     *            User's password (clear text)
     * @param roles
     *            Comma-delimited set of roles associated with this
     *            user
     */
    void addUser(
            String username, String password, String roles) {

        // Accumulate the list of roles for this user
        ArrayList<String> list = new ArrayList<String>();
        roles += ",";
        while (true) {
            int comma = roles.indexOf(',');
            if (comma < 0) {
                break;
            }
            String role = roles.substring(
                0, comma).trim();
            list.add(role);
            roles = roles.substring(comma + 1);
        }

        // Construct and cache the Principal for this user
        GenericPrincipal principal = new GenericPrincipal(
                this, username, password, list);
        this.principals.put(
            username, principal);

    }

    // ------------------------------------------------------
    // Protected Methods

    /**
     * Return a configured <code>Digester</code> to use for
     * processing the XML input file, creating a new one if necessary.
     * 
     * @return a configured <code>Digester</code> to use for
     *         processing the XML input file
     */
    protected synchronized Digester getDigester() {

        if (MemoryRealm.digester == null) {
            MemoryRealm.digester = new Digester();
            MemoryRealm.digester.setValidating(false);
            MemoryRealm.digester.addRuleSet(new MemoryRuleSet());
        }
        return (MemoryRealm.digester);

    }

    /**
     * Return a short Name for this Realm implementation.
     * 
     * @return a short Name for this Realm implementation.
     */
    protected String getName() {

        return (MemoryRealm.NAME);

    }

    /**
     * Return the password associated with the given principal's user
     * NAME.
     * 
     * @param username
     *            the user NAME.
     * @return the password associated with the given principal's user
     *         NAME.
     */
    protected String getPassword(
            String username) {

        GenericPrincipal principal =
                (GenericPrincipal) this.principals.get(username);
        if (principal != null) {
            return (principal.getPassword());
        } else {
            return (null);
        }

    }

    /**
     * Return the Principal associated with the given user NAME.
     * 
     * @param username
     *            the user NAME
     * @return the Principal associated with the given user NAME.
     */
    protected Principal getPrincipal(
            String username) {

        return (Principal) this.principals.get(username);

    }

    /**
     * Returns the principals for this realm.
     * 
     * @return The principals, keyed by user NAME (a String)
     */
    protected Map<String, GenericPrincipal> getPrincipals() {

        return this.principals;
    }

    // ------------------------------------------------------
    // Lifecycle Methods

    /**
     * Prepare for active use of the public methods of this Component.
     * 
     * @exception LifecycleException
     *                if this component detects a fatal error that
     *                prevents it from being started
     */
    public synchronized void start() throws LifecycleException {

        // Perform normal superclass initialization
        super.start();

        // Validate the existence of our database file
        File file = new File(
                this.pathname);
        if (!file.isAbsolute()) {
            file = new File(
                    System.getProperty("catalina.base"), this.pathname);
        }
        if (!file.exists() || !file.canRead()) {
            throw new LifecycleException(
                    MemoryRealm.sm.getString(
                        "memoryRealm.loadExist", file.getAbsolutePath()));
        }

        // Load the contents of the database file
        if (MemoryRealm.log.isDebugEnabled()) {
            MemoryRealm.log.debug(MemoryRealm.sm.getString(
                "memoryRealm.loadPath", file.getAbsolutePath()));
        }
        Digester digester = this.getDigester();
        try {
            synchronized (digester) {
                digester.push(this);
                digester.parse(file);
            }
        } catch (Exception e) {
            throw new LifecycleException(
                    "memoryRealm.readXml", e);
        } finally {
            digester.reset();
        }

    }

    /**
     * Gracefully shut down active use of the public methods of this
     * Component.
     * 
     * @exception LifecycleException
     *                if this component detects a fatal error that
     *                needs to be reported
     */
    public synchronized void stop() throws LifecycleException {

        // Perform normal superclass finalization
        super.stop();

        // No shutdown activities required

    }

}
